package appchain

import (
	"context"

	"github.com/meshplus/bitxhub-kit/storage"
	"github.com/sirupsen/logrus"

	"gitlab.33.cn/link33/sidecar/internal/port"
	"gitlab.33.cn/link33/sidecar/internal/txcrypto"
	"gitlab.33.cn/link33/sidecar/pkg/plugins"
	"gitlab.33.cn/link33/sidecar/tool"
)

type Manager struct {
	ctx     context.Context
	g       *tool.Group
	clients []plugins.Client
	store   storage.Storage
	cryptor txcrypto.Cryptor
	logger  logrus.FieldLogger
}

func NewManager(store storage.Storage, clients []plugins.Client, cryptor txcrypto.Cryptor, logger logrus.FieldLogger) *Manager {
	g, ctx := tool.WithContext(context.Background())
	return &Manager{
		ctx:     ctx,
		g:       g,
		clients: clients,
		store:   store,
		cryptor: cryptor,
		logger:  logger,
	}
}

func (m *Manager) Start() error {
	for _, client := range m.clients {
		m.g.Go(func() error {
			return client.Start()
		})
	}
	return m.g.Wait()
}

func (m *Manager) Stop() error {
	for _, client := range m.clients {
		m.g.Go(func() error {
			return client.Stop()
		})
	}
	return m.g.Wait()
}

func (m *Manager) Ports() []port.Port {
	return NewPorts(m.clients, m.cryptor, m.logger)
}
