package loggers

import (
	"github.com/sirupsen/logrus"

	"gitlab.33.cn/link33/sidecar/internal/repo"
	"gitlab.33.cn/link33/sidecar/pkg/log"
)

const (
	ApiServer   = "api_server"
	App         = "app"
	Manager     = "manager"
	RuleMgr     = "rule_mgr"
	AppchainMgr = "appchain_mgr"
	PeerMgr     = "peer_mgr"
	Router      = "router"
	Swarm       = "swarm"
	Syncer      = "syncer"
)

var w *loggerWrapper

type loggerWrapper struct {
	loggers map[string]*logrus.Entry
}

func InitializeLogger(config *repo.Config) {
	m := make(map[string]*logrus.Entry)
	m[ApiServer] = log.NewWithModule(ApiServer)
	m[ApiServer].Logger.SetLevel(log.ParseLevel(config.Log.Module.ApiServer))
	m[App] = log.NewWithModule(App)
	m[App].Logger.SetLevel(log.ParseLevel(config.Log.Level))
	m[AppchainMgr] = log.NewWithModule(AppchainMgr)
	m[AppchainMgr].Logger.SetLevel(log.ParseLevel(config.Log.Module.AppchainMgr))
	m[Router] = log.NewWithModule(Router)
	m[Router].Logger.SetLevel(log.ParseLevel(config.Log.Module.Router))
	m[Swarm] = log.NewWithModule(Swarm)
	m[Swarm].Logger.SetLevel(log.ParseLevel(config.Log.Module.Swarm))
	m[Syncer] = log.NewWithModule(Syncer)
	m[Syncer].Logger.SetLevel(log.ParseLevel(config.Log.Module.Syncer))
	m[PeerMgr] = log.NewWithModule(PeerMgr)
	m[PeerMgr].Logger.SetLevel(log.ParseLevel(config.Log.Module.PeerMgr))
	m[Manager] = log.NewWithModule(Manager)
	m[Manager].Logger.SetLevel(log.ParseLevel(config.Log.Module.Manager))
	m[RuleMgr] = log.NewWithModule(RuleMgr)
	m[RuleMgr].Logger.SetLevel(log.ParseLevel(config.Log.Module.Rule))
	w = &loggerWrapper{loggers: m}
}

func Logger(name string) logrus.FieldLogger {
	return w.loggers[name]
}
